VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CFileOp"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'---------------------------------------------
' The CFileOp class module - first version
'---------------------------------------------

Option Explicit

' this class exposes the following events
Event FileCopied(file As String, DestPath As String)

' private member variables
Private m_FileSpec As String
Private m_Filenames As Collection
Private m_Attributes As VbFileAttribute

' The FileSpec property (also includes the path)

Property Get FileSpec() As String
    FileSpec = m_FileSpec
End Property

Property Let FileSpec(ByVal newValue As String)
    ' reset the internal collection only if a new filespec is given
    If m_FileSpec <> newValue Then
        m_FileSpec = newValue
        Set m_Filenames = Nothing
    End If
End Property

' The path of source files
' this property derives and affects FileSpec, but doesn't
' correspond to any private member variable

Property Get Path() As String
    Path = GetPath(m_FileSpec)
End Property

Property Let Path(ByVal newValue As String)
    ' get the current Filespec, then substitute just the path
    FileSpec = MakeFilename(newValue, GetFileName(FileSpec))
End Property

' The attributes of the files that must be processed

Property Get Attributes() As VbFileAttribute
    Attributes = m_Attributes
End Property

Property Let Attributes(ByVal newValue As VbFileAttribute)
    ' reset the internal collection only if a new value is given
    If m_Attributes <> newValue Then
        m_Attributes = newValue
        Set m_Filenames = Nothing
    End If
End Property

' holds the list of all the files that match FileSpec
' plus any other file added by the client code
' this is a read-only property

Property Get Filenames() As Collection
    ' build the filelist "on demand", and only if necessary
    If m_Filenames Is Nothing Then ParseFileSpec
    Set Filenames = m_Filenames
End Property

' Parses a file specification and attributes
' and adds the resulting filename to the internal m_Filenames collection

Sub ParseFileSpec(Optional FileSpec As Variant, _
    Optional Attributes As VbFileAttribute)
        Dim file As String, Path As String
        ' provide a default for arguments
        If IsMissing(FileSpec) Then
            ' in this case we need a file specification
            If Me.FileSpec = "" Then Err.Raise 1001, , "FileSpec undefined"
            FileSpec = Me.FileSpec
            Attributes = Me.Attributes
        End If
        ' create the internal collection if necessary
        If m_Filenames Is Nothing Then
            Set m_Filenames = New Collection
        End If
        Path = GetPath(FileSpec)
        ' do the real parsing work here
        file = Dir$(FileSpec, Attributes)
        Do While Len(file)
            If Not Cancel Then m_Filenames.Add MakeFilename(Path, file)
            file = Dir$
        Loop
End Sub

' Copy all matching files

Sub Copy(DestPath As String)
    Dim var As Variant, file As String, dest As String
    On Error Resume Next
    For Each var In Filenames
        file = CStr(var)
        dest = MakeFilename(DestPath, GetFileName(file))
        FileCopy file, dest
        
        If Err = 0 Then
            RaiseEvent FileCopied(file, DestPath)
        Else
            Err.Clear
        End If
    Next
End Sub


' a support routine that parses a filename
' it is used internally, but it is also exposed as Public for convenience

Sub SplitFilename(ByVal CompleteName As String, Path As String, file As String, _
    Optional Extension As Variant)
        Dim i As Integer
        ' assume there isn't any embedded path
        Path = ""
        file = CompleteName
        ' backward search for a path delimiter
        For i = Len(file) To 1 Step -1
            If Mid$(file, i, 1) = "." And Not IsMissing(Extension) Then
                ' we have found an extension and the caller asked for it
                Extension = Mid$(file, i + 1)
                file = Left$(file, i - 1)
            ElseIf InStr(":\", Mid$(file, i, 1)) Then
                ' paths don't have a trailing backslash
                Path = Left$(file, i)
                If Right$(Path, 1) = "\" Then Path = Left$(Path, i - 1)
                file = Mid$(file, i + 1)
                Exit For
            End If
        Next
End Sub

' return the Path portion of a complete filename
' Note: uses SplitFilename

Function GetPath(ByVal CompleteFileName As String) As String
    SplitFilename CompleteFileName, GetPath, ""
End Function

' return the BaseFile portion of a complete filename
' Note: uses SplitFilename

Function GetBaseName(ByVal CompleteFileName As String) As String
    ' you need to pass a dummy extension argument
    SplitFilename CompleteFileName, "", GetBaseName, ""
End Function

' return the FileName portion of a complete filename
' Note: uses SplitFilename

Function GetFileName(ByVal CompleteFileName As String) As String
    ' don't pass any extension argument
    SplitFilename CompleteFileName, "", GetFileName
End Function

' return the Extension portion of a complete filename
' Note: uses SplitFilename

Function GetExtension(ByVal CompleteFileName As String) As String
    SplitFilename CompleteFileName, "", "", GetExtension
End Function

' a support routine that builds a filename
' it is used internally, but it is also exposed as Public for convenience

Function MakeFilename(ByVal Path As String, ByVal FileName As String, _
    Optional Extension As String) As String
        Dim result As String
        If Path <> "" Then
            ' path may or may not include a trailing backslash
            result = Path & IIf(Right$(Path, 1) <> "\", "\", "")
        End If
        result = result & FileName
        If Extension <> "" Then
            ' extension may or may not include a dot
            result = result & IIf(Left$(Extension, 1) = ".", ".", "") & Extension
        End If
        MakeFilename = result
End Function


