Attribute VB_Name = "Drives"
'---------------------------------------------------------
' PROGRAMMING VB6 ROUTINE LIBRARY - (C) Francesco Balena
'
' A collection of disk routines
'---------------------------------------------------------

' check that a drive contains a disk with a given volume label
' if it doesn't, asks the user to retry or cancel the operation
' returns True if the volume label matches, False otherwise

Function CheckDisk(ByVal Drive As String, VolumeLabel As String) As Boolean
    Dim saveDir As String, answer As Integer
    On Error GoTo Error_Handler
    Drive = Left$(Drive, 1)
    ' try to make that drive the current one
    saveDir = CurDir$
    ' next statement is likely to fire an error
    ChDrive Drive
    ' return True if the label matches, False otherwise
    CheckDisk = (StrComp(Dir$(Drive & ":\*.*", vbVolume), _
        VolumeLabel, vbTextCompare) = 0)
    ' restore the original current drive
    ChDrive saveDir
    Exit Function
Error_Handler:
    If Err <> 68 And Err <> 71 Then
        ' if Device unavailable/Disk Not Ready, report it to the caller
        Err.Raise Err.number, Err.source, Err.Description
    End If
    ' if it is a floppy disk, give the user a second chance
    If InStr(1, "AB", Drive, vbTextCompare) Then
        answer = MsgBox("Please enter a disk in drive " & Drive, _
            vbExclamation + vbRetryCancel)
        If answer = vbRetry Then Resume
    End If
    ' else Resume with the next statement (returns False)
    Resume Next
End Function

Option Explicit

