Attribute VB_Name = "EnumWindowsAPI"
Option Explicit

Private Declare Function EnumWindows Lib "user32" (ByVal lpEnumFunc As Long, ByVal lParam As Long) As Long
Private Declare Function EnumChildWindows Lib "user32" (ByVal hWndParent As Long, ByVal lpEnumFunc As Long, ByVal lParam As Long) As Long
Private Declare Function GetClassName Lib "user32" Alias "GetClassNameA" (ByVal hWnd As Long, ByVal lpClassName As String, ByVal nMaxCount As Long) As Long
Private Declare Function GetWindowText Lib "user32" Alias "GetWindowTextA" (ByVal hWnd As Long, ByVal lpString As String, ByVal cch As Long) As Long

' An array of Long holding the handle of all child windows.
Dim windows() As Long
' The number of elements in the array.
Dim windowsCount As Long

' Return an array of Long holding the handles of all the child windows
' of a given window. If hWnd = 0 it returns all the top-level windows.

Function ChildWindows(ByVal hWnd As Long) As Long()
    windowsCount = 0
    If hWnd Then
        EnumChildWindows hWnd, AddressOf EnumWindows_CBK, 1
    Else
        EnumWindows AddressOf EnumWindows_CBK, 1
    End If
    ' Trim uninitialized elements and return to caller.
    ReDim Preserve windows(windowsCount) As Long
    ChildWindows = windows()
End Function

' The callback routine, common to both EnumWindows and EnumChildWindows.

Function EnumWindows_CBK(ByVal hWnd As Long, ByVal lParam As Long) As Long
    ' Make room in the array, if necessary.
    If windowsCount = 0 Then
        ReDim windows(100) As Long
    ElseIf windowsCount >= UBound(windows) Then
        ReDim Preserve windows(windowsCount + 100) As Long
    End If
    ' Store the new item.
    windowsCount = windowsCount + 1
    windows(windowsCount) = hWnd
    ' Return 1 to continue enumeration.
    EnumWindows_CBK = 1
End Function

' Return the caption/text of a window.

Function WindowText(ByVal hWnd As Long) As String
    Dim buffer As String * 256, length As Long
    length = GetWindowText(hWnd, buffer, Len(buffer))
    WindowText = Left$(buffer, length)
End Function

Function WindowClassName(ByVal hWnd As Long) As String
    Dim buffer As String * 256, length As Long
    length = GetClassName(hWnd, buffer, Len(buffer))
    WindowClassName = Left$(buffer, length)
End Function

Function WindowDescription(ByVal hWnd As Long) As String
    Dim text As String
    text = WindowText(hWnd)
    
    WindowDescription = "[" & Right$("0000000" & Hex$(hWnd), 8) & "] " _
        & WindowClassName(hWnd)
    If Len(text) Then
        WindowDescription = WindowDescription & " - """ & text & """"
    End If
End Function
