VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Mouse"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Mouse routines"
'---------------------------------------------------------
' $CLSDESCR Mouse routines
'---------------------------------------------------------

Option Explicit

Private Type POINTAPI
    X As Long
    Y As Long
End Type
Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private Declare Function GetAsyncKeyState Lib "user32" (ByVal vKey As Long) As Integer
Private Declare Function GetSystemMetrics Lib "user32" (ByVal nIndex As Long) As Long
Private Declare Function ShowCursor Lib "user32" (ByVal bShow As Long) As Long
Private Declare Function SetCursorPos Lib "user32" (ByVal X As Long, ByVal Y As Long) As Long
Private Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long
Private Declare Function ScreenToClient Lib "user32" (ByVal hWnd As Long, lpPoint As POINTAPI) As Long
Private Declare Function ClientToScreen Lib "user32" (ByVal hWnd As Long, lpPoint As POINTAPI) As Long
Private Declare Function ClipCursor Lib "user32" (lpRect As Any) As Long
Private Declare Function GetClientRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function SetCapture Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function ReleaseCapture Lib "user32" () As Long
Private Declare Function GetCapture Lib "user32" () As Long
Private Declare Function WindowFromPoint Lib "user32" (ByVal xPoint As Long, ByVal yPoint As Long) As Long

' $DESCR Return the state of the Left button.

Function LeftButton() As Boolean
Attribute LeftButton.VB_Description = "Return the state of the Left button."
    LeftButton = (GetAsyncKeyState(vbKeyLButton) And &H8000)
End Function

' $DESCR Return the state of the Right button.

Function RightButton() As Boolean
Attribute RightButton.VB_Description = "Return the state of the Right button."
    RightButton = (GetAsyncKeyState(vbKeyRButton) And &H8000)
End Function

' $DESCR Return the state of the Middle button.

Function MiddleButton() As Boolean
Attribute MiddleButton.VB_Description = "Return the state of the Middle button."
    MiddleButton = (GetAsyncKeyState(vbKeyMButton) And &H8000)
End Function

' $DESCR Return the state of the mouse buttons as a bit-coded value.
' $DESCR (same format as the Button argument of a Mousexxxx event procedure.)

Function MouseButton() As Integer
Attribute MouseButton.VB_Description = "Return the state of the mouse buttons as a bit-coded value. (same format as the Button argument of a Mousexxxx event procedure.)."
    If GetAsyncKeyState(vbKeyLButton) < 0 Then
        MouseButton = 1
    End If
    If GetAsyncKeyState(vbKeyRButton) < 0 Then
        MouseButton = MouseButton Or 2
    End If
    If GetAsyncKeyState(vbKeyMButton) < 0 Then
        MouseButton = MouseButton Or 4
    End If
End Function

' $DESCR Return the number of mouse buttons.

Function MouseButtonsNumber() As Integer
Attribute MouseButtonsNumber.VB_Description = "Return the number of mouse buttons."
    Const SM_CMOUSEBUTTONS = 43
    MouseButtonsNumber = GetSystemMetrics(SM_CMOUSEBUTTONS)
End Function

' $DESCR Show the mouse cursor.

Sub MouseHide()
Attribute MouseHide.VB_Description = "Show the mouse cursor."
    ShowCursor False
End Sub

' $DESCR Hide the mouse cursor.

Sub MouseShow()
Attribute MouseShow.VB_Description = "Hide the mouse cursor."
    ShowCursor True
End Sub

' $DESCR Get mouse X coordinates (screen coordinates).
' $DESCR If a window handle is passed, the result are client area coordinates
' $DESCR relative to that window.

Function MouseX(Optional ByVal hWnd As Long) As Long
Attribute MouseX.VB_Description = "Get mouse X coordinates (screen coordinates). If a window handle is passed, the result are client area coordinates relative to that window."
    Dim lpPoint As POINTAPI
    GetCursorPos lpPoint
    If hWnd Then ScreenToClient hWnd, lpPoint
    MouseX = lpPoint.X
End Function

' $DESCR Get mouse Y coordinates (screen coordinates).
' $DESCR If a window handle is passed, the result are client area coordinates
' $DESCR relative to that window.

Function MouseY(Optional ByVal hWnd As Long) As Long
Attribute MouseY.VB_Description = "Get mouse Y coordinates (screen coordinates). If a window handle is passed, the result are client area coordinates relative to that window."
    Dim lpPoint As POINTAPI
    GetCursorPos lpPoint
    If hWnd Then ScreenToClient hWnd, lpPoint
    MouseY = lpPoint.Y
End Function

' $DESCR Mouse mouse cursor. If a window handle is passed, the coordinates
' $DESCR are taken as relative to that window's client area.

Sub MoveMouseCursor(ByVal X As Long, ByVal Y As Long, Optional ByVal hWnd As Long)
Attribute MoveMouseCursor.VB_Description = "Mouse mouse cursor. If a window handle is passed, the coordinates are taken as relative to that window's client area."
    If hWnd = 0 Then
        SetCursorPos X, Y
    Else
        Dim lpPoint As POINTAPI
        lpPoint.X = X
        lpPoint.Y = Y
        ClientToScreen hWnd, lpPoint
        SetCursorPos lpPoint.X, lpPoint.Y
    End If
End Sub

' $DESCR Clip the mouse cursor to a given rectangle.

Sub ClipMouse(ByVal Left As Long, ByVal Top As Long, ByVal Right As Long, ByVal Bottom As Long)
Attribute ClipMouse.VB_Description = "Clip the mouse cursor to a given rectangle."
    Dim lpRect As RECT
    lpRect.Left = Left
    lpRect.Top = Top
    lpRect.Right = Right
    lpRect.Bottom = Bottom
    ClipCursor lpRect
End Sub

' $DESCR Clip mouse to a given window.

Sub ClipMouseToWindow(ByVal hWnd As Long)
Attribute ClipMouseToWindow.VB_Description = "Clip mouse to a given window."
    Dim lpPoint As POINTAPI, lpRect As RECT
    ' Retrieve the coordinates of the upper-left corner of the window.
    ClientToScreen hWnd, lpPoint
    ' Get the client screen rectangle.
    GetClientRect hWnd, lpRect
    ' Manually convert the rectangle's elements to screen coordinates.
    lpRect.Left = lpRect.Left + lpPoint.X
    lpRect.Top = lpRect.Top + lpPoint.Y
    lpRect.Right = lpRect.Right + lpPoint.X
    lpRect.Bottom = lpRect.Bottom + lpPoint.Y
    ' Enforce the clipping.
    ClipCursor lpRect
End Sub

' $DESCR Clear the mouse clip area.

Sub ClearClipMouse()
Attribute ClearClipMouse.VB_Description = "Clear the mouse clip area."
    ClipCursor ByVal 0&
End Sub

' $DESCR Set mouse capture.Pass zero to release mouse capture.

Sub SetMouseCapture(ByVal hWnd As Long)
Attribute SetMouseCapture.VB_Description = "Set mouse capture.Pass zero to release mouse capture."
    If hWnd Then
        SetCapture hWnd
    Else
        ReleaseCapture
    End If
End Sub

' $DESCR Get the handle of the window that has the mouse capture.

Function GetMouseCapture() As Long
Attribute GetMouseCapture.VB_Description = "Get the handle of the window that has the mouse capture."
    GetMouseCapture = GetCapture()
End Function

' $DESCR Return the handle of the window under the mouse cursor.

Function WindowFromMouse() As Long
Attribute WindowFromMouse.VB_Description = "Return the handle of the window under the mouse cursor."
    Dim lpPoint As POINTAPI
    GetCursorPos lpPoint
    WindowFromMouse = WindowFromPoint(lpPoint.X, lpPoint.Y)
End Function

