VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "TextBoxAPI"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Routines for TextBox controls"
'---------------------------------------------------------
' $CLSDESCR Routines for TextBox controls
'---------------------------------------------------------

Option Explicit

Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function SendMessageByVal Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Const EM_CANUNDO = &HC6
Attribute EM_CANUNDO.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_UNDO = &HC7
Attribute EM_UNDO.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_EMPTYUNDOBUFFER = &HCD
Attribute EM_EMPTYUNDOBUFFER.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_GETLINECOUNT = &HBA
Attribute EM_GETLINECOUNT.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_GETFIRSTVISIBLELINE = &HCE
Attribute EM_GETFIRSTVISIBLELINE.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_LINEFROMCHAR = &HC9
Attribute EM_LINEFROMCHAR.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_LINEINDEX = &HBB
Attribute EM_LINEINDEX.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_LINELENGTH = &HC1
Attribute EM_LINELENGTH.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_FMTLINES = &HC8
Attribute EM_FMTLINES.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_GETLINE = &HC4
Attribute EM_GETLINE.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_GETRECT = &HB2
Attribute EM_GETRECT.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_SCROLL = &HB5
Attribute EM_SCROLL.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_SCROLLCARET = &HB7
Attribute EM_SCROLLCARET.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_SETRECT = &HB3
Attribute EM_SETRECT.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_SETTABSTOPS = &HCB
Attribute EM_SETTABSTOPS.VB_VarDescription = "Return True if the most recent edit operation can be undone."
Const EM_LINESCROLL = &HB6
Attribute EM_LINESCROLL.VB_VarDescription = "Return True if the most recent edit operation can be undone."

'-------------------------------------------
' Undo operations
'-------------------------------------------

' $DESCR Return True if the most recent edit operation can be undone.

Function CanUndo(tb As Object) As Boolean
Attribute CanUndo.VB_Description = "Return True if the most recent edit operation can be undone."
    CanUndo = SendMessageByVal(tb.hWnd, EM_CANUNDO, 0, 0)
End Function

' $DESCR Undo the most recent edit operation.

Sub Undo(tb As Object)
Attribute Undo.VB_Description = "Undo the most recent edit operation."
    SendMessageByVal tb.hWnd, EM_UNDO, 0, 0
End Sub

' $DESCR Empty the undo buffer.

Sub EmptyUndoBuffer(tb As Object)
Attribute EmptyUndoBuffer.VB_Description = "Empty the undo buffer."
    SendMessageByVal tb.hWnd, EM_EMPTYUNDOBUFFER, 0, 0
End Sub

'-------------------------------------------
' Multi-line controls
'-------------------------------------------

' $DESCR Return the number of lines in the control.

Function GetLineCount(tb As Object) As Long
Attribute GetLineCount.VB_Description = "Return the number of lines in the control."
    GetLineCount = SendMessageByVal(tb.hWnd, EM_GETLINECOUNT, 0, 0)
End Function

' $DESCR Return the index of the first visible line
' $DESCR (0 for the first text line in the control).
' $DESCR When applied to a single-line control, return the
' $DESCR index of the first visible character
' $DESCR (0 for the first character in the control).

Function GetFirstVisibleLine(tb As Object) As Long
Attribute GetFirstVisibleLine.VB_Description = "Return the index of the first visible line (0 for the first text line in the control). When applied to a single-line control, return the index of the first visible character (0 for the first character in the control)."
    GetFirstVisibleLine = SendMessageByVal(tb.hWnd, EM_GETFIRSTVISIBLELINE, 0, 0)
End Function

' $DESCR Return the number of the line that contains the specified character.
' $DESCR Both line and character numbers are zero-based.

Function LineFromChar(tb As Object, ByVal charIndex As Long) As Long
Attribute LineFromChar.VB_Description = "Return the number of the line that contains the specified character. Both line and character numbers are zero-based."
    LineFromChar = SendMessageByVal(tb.hWnd, EM_LINEFROMCHAR, charIndex, 0)
End Function

' $DESCR Return the character offset of the first character of a line.

Function LineIndex(tb As Object, ByVal lineNum As Long) As Long
Attribute LineIndex.VB_Description = "Return the character offset of the first character of a line."
    LineIndex = SendMessageByVal(tb.hWnd, EM_LINEINDEX, lineNum, 0)
End Function

' $DESCR Return the length of the specified line.

Function LineLength(tb As Object, ByVal lineNum As Long) As Long
Attribute LineLength.VB_Description = "Return the length of the specified line."
    Dim charOffset As Long
    ' Retrieve the character offset of the first character of the line.
    charOffset = SendMessageByVal(tb.hWnd, EM_LINEINDEX, lineNum, 0)
    ' Now it is possible to retrieve the length of the line.
    LineLength = SendMessageByVal(tb.hWnd, EM_LINELENGTH, charOffset, 0)
End Function

' $DESCR Return the specified line.

Function GetLine(tb As Object, ByVal lineNum As Long) As String
Attribute GetLine.VB_Description = "Return the specified line."
    Dim charOffset As Long, lineLen As Long
    ' Retrieve the character offset of the first character of the line.
    charOffset = SendMessageByVal(tb.hWnd, EM_LINEINDEX, lineNum, 0)
    ' Now it is possible to retrieve the length of the line.
    lineLen = SendMessageByVal(tb.hWnd, EM_LINELENGTH, charOffset, 0)
    ' Extract the line text.
    GetLine = Mid$(tb.text, charOffset + 1, lineLen)
End Function

' $DESCR Get the line/column coordinates of a given character (both are zero-based).
' $DESCR If charIndex is negative, it returns the coordinates of the caret

Sub GetLineColumn(tb As Object, ByVal charIndex As Long, line As Long, column As Long)
Attribute GetLineColumn.VB_Description = "Get the line/column coordinates of a given character (both are zero-based). If charIndex is negative, it returns the coordinates of the caret."
    If charIndex < 0 Then charIndex = tb.SelStart
    ' Get the line number.
    line = SendMessageByVal(tb.hWnd, EM_LINEFROMCHAR, charIndex, 0)
    ' Get the column number by subtracting the line's start
    ' index from the caret position
    column = tb.SelStart - SendMessageByVal(tb.hWnd, EM_LINEINDEX, line, 0)
End Sub

' $DESCR Return an array with all the lines in the control.
' $DESCR If the second optional argument is True, the trailing CR-LF is preserved.

Function GetAllLines(tb As Object, Optional KeepHardLineBreaks As Boolean) As String()
Attribute GetAllLines.VB_Description = "Return an array with all the lines in the control. If the second optional argument is True, the trailing CR-LF is preserved."
    Dim result() As String, i As Long
    ' Activate soft line breaks. A soft line break is marked by the
    ' CR-Cr-LF sequence.
    SendMessageByVal tb.hWnd, EM_FMTLINES, True, 0
    ' Retrieve all the lines in one operation. This operation will leave
    ' a trailing CR character for soft line breaks.
    result() = Split(tb.text, vbCrLf)
    ' We need a loop to trim the trailing CR character. If the second
    ' argument is true, we need to manually add a CR-LF pair to all
    ' the lines that don't contain such trailing CR char.
    For i = 0 To UBound(result)
        If Right$(result(i), 1) = vbCr Then
            result(i) = Left$(result(i), Len(result(i)) - 1)
        ElseIf KeepHardLineBreaks Then
            result(i) = result(i) & vbCrLf
        End If
    Next
    ' Deactivate soft line breaks.
    SendMessageByVal tb.hWnd, EM_FMTLINES, False, 0
    GetAllLines = result()
End Function

' $DESCR Get the formatting rectangle.

Sub GetRect(tb As Object, Left As Long, Top As Long, Right As Long, Bottom As Long)
Attribute GetRect.VB_Description = "Get the formatting rectangle."
    Dim lpRect As RECT
    SendMessage tb.hWnd, EM_GETRECT, 0, lpRect
    Left = lpRect.Left
    Top = lpRect.Top
    Right = lpRect.Right
    Bottom = lpRect.Bottom
End Sub

' $DESCR Set the formatting rectangle and refresh the control.

Sub SetRect(tb As Object, ByVal Left As Long, ByVal Top As Long, ByVal Right As Long, ByVal Bottom As Long)
Attribute SetRect.VB_Description = "Set the formatting rectangle and refresh the control."
    Dim lpRect As RECT
    lpRect.Left = Left
    lpRect.Top = Top
    lpRect.Right = Right
    lpRect.Bottom = Bottom
    SendMessage tb.hWnd, EM_SETRECT, 0, lpRect
End Sub

' $DESCR Scroll the contents of the control.
' $DESCR Positive values scroll left and up, negative values
' $DESCR scroll right and down.

Sub LineScroll(tb As Object, ByVal HorizScroll As Long, ByVal VertScroll As Long)
Attribute LineScroll.VB_Description = "Scroll the contents of the control. Positive values scroll left and up, negative values scroll right and down."
    SendMessageByVal tb.hWnd, EM_LINESCROLL, HorizScroll, VertScroll
End Sub

' $DESCR Ensures that the caret is visible.

Sub ScrollCaret(tb As Object)
Attribute ScrollCaret.VB_Description = "Ensures that the caret is visible."
    SendMessageByVal tb.hWnd, EM_SCROLLCARET, 0, 0
End Sub

' $DESCR Set tab stops. Each element of the array is expressed in dialog units,
' $DESCR where each dialog unit is 1/4 of the average character width.

Sub TBSetTabStops(tb As Object, tabStops() As Long)
Attribute TBSetTabStops.VB_Description = "Set tab stops. Each element of the array is expressed in dialog units, where each dialog unit is 1/4 of the average character width."
    Dim numEls As Long
    numEls = UBound(tabStops) - LBound(tabStops) + 1
    SendMessage tb.hWnd, EM_SETTABSTOPS, numEls, tabStops(LBound(tabStops))
End Sub

' $DESCR Set the tab stop distance, expressed in dialog units.

Sub TBSetTabStopDistance(tb As Object, ByVal distance As Long)
Attribute TBSetTabStopDistance.VB_Description = "Set the tab stop distance, expressed in dialog units."
    SendMessage tb.hWnd, EM_SETTABSTOPS, 1, distance
End Sub

