VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Trig"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "TRIG routines"
'---------------------------------------------------------
' $CLSDESCR TRIG routines
'---------------------------------------------------------

Option Explicit

'-----------------------------------------
' Additional Trig functions
'-----------------------------------------

' $DESCR Cotangent

Function Cot(Radians As Double) As Double
Attribute Cot.VB_Description = "Cotangent."
    Cot = 1 / Tan(Radians)
End Function

' $DESCR Secant

Function Sec(Radians As Double) As Double
Attribute Sec.VB_Description = "Secant."
    Sec = 1 / Cos(Radians)
End Function

' $DESCR Cosecant

Function Csc(Radians As Double) As Double
Attribute Csc.VB_Description = "Cosecant."
    Csc = 1 / Sin(Radians)
End Function

'-----------------------------------------
' Inverse Trig functions
'-----------------------------------------

' $DESCR Arc Sine (error if Number is outside the range [-1,1])

Function ASin(ByVal Number As Double) As Double
Attribute ASin.VB_Description = "Arc Sine (error if Number is outside the range [-1,1])."
    If Abs(Number) <> 1 Then
        ASin = Atn(Number / Sqr(1 - Number * Number))
    Else
        ASin = 1.5707963267949 * Sgn(Number)
    End If
End Function

' $DESCR Arc Cosine (error if Number is outside the range [-1,1])

Function ACos(ByVal Number As Double) As Double
Attribute ACos.VB_Description = "Arc Cosine (error if Number is outside the range [-1,1])."
    If Abs(Number) <> 1 Then
        ACos = 1.5707963267949 - Atn(Number / Sqr(1 - Number * Number))
    Else
        ACos = 3.14159265358979 * Sgn(Number)
    End If
End Function

' $DESCR Arc Tangent of Y/X (returns values in all four quadrants).

Function Atn2(X As Double, Y As Double) As Double
Attribute Atn2.VB_Description = "Arc Tangent of Y/X (returns values in all four quadrants)."
    If X = 0 Then
        Atn2 = Sgn(Y) * 1.5707963267949
    ElseIf X > 0 Then
        Atn2 = Atn(Y / X)
    Else
        Atn2 = Atn(Y / X) + 3.14159265358979 * (Y < 0)
    End If
End Function

' $DESCR Arc Cotangent (error if Number is zero)

Function ACot(Number As Double) As Double
Attribute ACot.VB_Description = "Arc Cotangent (error if Number is zero)."
    ACot = Atn(1 / Number)
End Function

' $DESCR Arc Secant (error if Number is inside the range [-1,1]).

Function ASec(Number As Double) As Double
Attribute ASec.VB_Description = "Arc Secant (error if Number is inside the range [-1,1])."
    If Abs(Number) <> 1 Then
        ASec = 1.5707963267949 - Atn((1 / Number) / Sqr(1 - 1 / (Number * Number)))
    Else
        ASec = 3.14159265358979 * Sgn(Number)
    End If
End Function

' $DESCR Arc Cosecant (error if Number is inside the range [-1,1]).

Function ACsc(Number As Double) As Double
Attribute ACsc.VB_Description = "Arc Cosecant (error if Number is inside the range [-1,1])."
    If Abs(Number) <> 1 Then
        ACsc = Atn((1 / Number) / Sqr(1 - 1 / (Number * Number)))
    Else
        ACsc = 1.5707963267949 * Sgn(Number)
    End If
End Function

'-----------------------------------------
' Conversion functions
'-----------------------------------------

' $DESCR Convert from carthesian to polar coordinates
' $DEPENDENCIES: Atn2

Sub XYToPolar(X As Double, Y As Double, angle As Double, radius As Double)
Attribute XYToPolar.VB_Description = "Convert from carthesian to polar coordinates."
    angle = Atn2(X, Y)
    radius = Sqr(X * X + Y * Y)
End Sub

' $DESCR Convert from polar to carthesian coordinates.

Sub PolarToXY(angle As Double, radius As Double, X As Double, Y As Double)
Attribute PolarToXY.VB_Description = "Convert from polar to carthesian coordinates."
    X = radius * Cos(angle)
    Y = radius * Sin(angle)
End Sub

'-----------------------------------------
' Hyperbolic functions
'-----------------------------------------

' $DESCR Hyperbolic Sine

Function SinH(Number As Double) As Double
Attribute SinH.VB_Description = "Hyperbolic Sine."
    Dim temp As Double
    temp = Exp(Number)
    SinH = (temp - 1 / temp) / 2
End Function

' $DESCR Hyperbolic Cosine

Function CosH(Number As Double) As Double
Attribute CosH.VB_Description = "Hyperbolic Cosine."
    Dim temp As Double
    temp = Exp(Number)
    CosH = (temp + 1 / temp) / 2
End Function

' $DESCR Hyperbolic Tangent

Function TanH(Number As Double) As Double
Attribute TanH.VB_Description = "Hyperbolic Tangent."
    Dim temp As Double
    temp = Exp(Number)
    TanH = (temp - 1 / temp) / (temp + 1 / temp)
End Function

' $DESCR Hyperbolic Cotangent

Function CotH(Number As Double) As Double
Attribute CotH.VB_Description = "Hyperbolic Cotangent."
    Dim temp As Double
    temp = Exp(Number)
    CotH = (temp + 1 / temp) / (temp - 1 / temp)
End Function

' $DESCR Hyperbolic Secant

Function SecH(Number As Double) As Double
Attribute SecH.VB_Description = "Hyperbolic Secant."
    Dim temp As Double
    temp = Exp(Number)
    SecH = 2 / (temp + 1 / temp)
End Function

' $DESCR Hyperbolic Cosecant

Function CscH(Number As Double) As Double
Attribute CscH.VB_Description = "Hyperbolic Cosecant."
    Dim temp As Double
    temp = Exp(Number)
    CscH = 2 / (temp - 1 / temp)
End Function

'-----------------------------------------
' Inverse Hyperbolic functions
'-----------------------------------------

' $DESCR Hyperbolic Arc Sine

Function ASinH(Number As Double) As Double
Attribute ASinH.VB_Description = "Hyperbolic Arc Sine."
    ASinH = Log(Number + Sqr(Number * Number + 1))
End Function

' $DESCR Hyperbolic Arc Cosine (error if Number is inside the range [-1,1]).

Function ACosH(Number As Double) As Double
Attribute ACosH.VB_Description = "Hyperbolic Arc Cosine (error if Number is inside the range [-1,1])."
    ACosH = Log(Number + Sqr(Number * Number - 1))
End Function

' $DESCR Hyperbolic Arc Tangent (error if Number is zero).

Function ATanH(Number As Double) As Double
Attribute ATanH.VB_Description = "Hyperbolic Arc Tangent (error if Number is zero)."
    ATanH = Log((1 / Number + 1) / (1 / Number - 1)) / 2
End Function

' $DESCR Hyperbolic Arc Cotangent.

Function ACotH(Number As Double) As Double
Attribute ACotH.VB_Description = "Hyperbolic Arc Cotangent."
    ACotH = Log((Number + 1) / (Number - 1)) / 2
End Function

' $DESCR Hyperbolic Arc Secant (error if Number is outside the range [-1,1]).

Function ASecH(Number As Double) As Double
Attribute ASecH.VB_Description = "Hyperbolic Arc Secant (error if Number is outside the range [-1,1])."
    ASecH = Log((Sqr(1 - Number * Number) + 1) / Number)
End Function

' $DESCR Hyperbolic Arc Cosecant

Function ACscH(Number As Double) As Double
Attribute ACscH.VB_Description = "Hyperbolic Arc Cosecant."
    ACscH = Log((Sgn(Number) * Sqr(1 + Number * Number) + 1) / Number)
End Function

