VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "MsgHook"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Event BeforeMessage(hWnd As Long, uMsg As Long, wParam As Long, lParam As Long, retValue As Long, Cancel As Boolean)
Event AfterMessage(ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long, retValue As Long)

' member variable for hWnd property
Private m_hWnd As Long
' member variable for Enabled property
Private m_Enabled As Boolean

' True if the component should raise event in its owner

Public Property Get Enabled() As Boolean
    Enabled = m_Enabled
End Property

Public Property Let Enabled(ByVal newValue As Boolean)
    m_Enabled = newValue
End Property

' The handle of the window being subclassed (read-only).

Property Get hWnd() As Long
    hWnd = m_hWnd
End Property

' Start the subclassing.
' You can stop subclassing by passing a null handle.

Sub StartSubclass(ByVal hWnd As Long)
    ' Minimal error trapping
    If hWnd <> 0 And IsWindow(hWnd) = 0 Then
        Err.Raise 1001, , "Invalid window handle"
    ElseIf hWnd = m_hWnd Then
        ' Nothing to do
        Exit Sub
    End If
    
    ' Terminate current subclassing, if necessary
    If m_hWnd Then StopSubclass
    ' Store arguments in member variables.
    m_hWnd = hWnd
   
    ' Add a new item to the list of subclassed window.
    If m_hWnd Then HookWindow Me, m_hWnd
End Sub

' Stop the subclassing.

Sub StopSubclass()
    ' Delete this item from the list of subclassed window.
    If m_hWnd Then UnhookWindow Me
End Sub

' This Friend procedure is called when a message is sent to this window.

Friend Function WndProc(ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long, ByVal oldWindowProc As Long) As Long
    Dim retValue As Long, Cancel As Boolean
    
    If Not Enabled Then
        ' If subclassing is disabled, just call the original procedure.
        retValue = CallWindowProc(oldWindowProc, hWnd, uMsg, wParam, lParam)
    Else
        ' Call the application before processing the message.
        RaiseEvent BeforeMessage(hWnd, uMsg, wParam, lParam, retValue, Cancel)
        If Not Cancel Then
            ' Call the original window procedure, unless the program canceled it.
            retValue = CallWindowProc(oldWindowProc, hWnd, uMsg, wParam, lParam)
            ' Call the application after processing the message.
            RaiseEvent AfterMessage(hWnd, uMsg, wParam, lParam, retValue)
        End If
    End If
    
    ' Return the value to Windows.
    WndProc = retValue
    
End Function

' ------------------------------------------------
' Private procedures
' ------------------------------------------------

Private Sub Class_Initialize()
    ' Initialize the Enabled property.
    Enabled = True
End Sub

' This event fires also when the client program is abruptly Ended.

Private Sub Class_Terminate()
    StopSubclass
End Sub
