Attribute VB_Name = "WindowProc"
Option Explicit

Declare Function IsWindow Lib "user32" (ByVal hWnd As Long) As Long
Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal ndx As Long, ByVal newValue As Long) As Long
Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hWnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

' This is used with the SetWindowLong API function.
Const GWL_WNDPROC = -4
' The message all windows receive before being closed.
Const WM_DESTROY = &H2

' --------------------------------------------

Type WindowInfoUDT
    hWnd As Long            ' Handle of window being subclassed
    oldWndProc As Long      ' Address of original window procedure
    obj As MsgHook          ' The MsgHook object serving this window
End Type
    
' This array stores info on subclassed windows.
Dim WindowInfo() As WindowInfoUDT
' This is the number of elements in the array.
Dim WindowInfoCount As Long

'------------------------------------------------------
' The subclassing routines
'------------------------------------------------------

' Start the subclassing of a window.

Sub HookWindow(obj As MsgHook, ByVal hWnd As Long)
    
    ' Make room in the array, if necessary
    If WindowInfoCount = 0 Then
        ReDim WindowInfo(10) As WindowInfoUDT
    ElseIf WindowInfoCount > UBound(WindowInfo) Then
        ReDim Preserve WindowInfo(WindowInfoCount + 9) As WindowInfoUDT
    End If
    WindowInfoCount = WindowInfoCount + 1
    
    ' Store data in the array, and start the subclassing of this window
    With WindowInfo(WindowInfoCount)
        .hWnd = hWnd
        Set .obj = obj
        .oldWndProc = SetWindowLong(hWnd, GWL_WNDPROC, AddressOf WndProc)
    End With
    
End Sub

' Stop the subclassing of the window associated to an object.

Sub UnhookWindow(obj As MsgHook)
    Dim i As Long, objPointer As Long
    
    For i = 1 To WindowInfoCount
        If WindowInfo(i).obj Is obj Then
            ' Stop subclassing.
            SetWindowLong WindowInfo(i).hWnd, GWL_WNDPROC, WindowInfo(i).oldWndProc
            ' Remove this element from the array.
            WindowInfo(i) = WindowInfo(WindowInfoCount)
            WindowInfo(WindowInfoCount) = WindowInfo(0)
            WindowInfoCount = WindowInfoCount - 1
            Exit For
        End If
    Next
End Sub

' The custom window procedure

Function WndProc(ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    Dim i As Long, obj As MsgHook
    
    ' Search this handle in the array.
    For i = 1 To WindowInfoCount
        If WindowInfo(i).hWnd = hWnd Then
            ' The client program is not in break mode.
            ' Notify the object that a message has arrived.
            Set obj = WindowInfo(i).obj
            WndProc = obj.WndProc(hWnd, uMsg, wParam, lParam, WindowInfo(i).oldWndProc)
            
            ' If we've got a WM_DESTROY message, the window is about to be closed
            ' therefore there is no point in keeping this object in the array.
            If uMsg = WM_DESTROY Then
                WindowInfo(i).obj.StopSubclass
            End If
            
            ' Nothing else to do (this version of the component
            ' doesn't support multiple objects that subclass the same window.
            Exit For
        End If
    Next
End Function

