Attribute VB_Name = "MouseAPI"
Option Explicit

Type POINTAPI
    X As Long
    Y As Long
End Type
Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private Declare Function GetAsyncKeyState Lib "user32" (ByVal vKey As Long) As Integer
Private Declare Function GetSystemMetrics Lib "user32" (ByVal nIndex As Long) As Long
Private Declare Function ShowCursor Lib "user32" (ByVal bShow As Long) As Long
Private Declare Function SetCursorPos Lib "user32" (ByVal X As Long, ByVal Y As Long) As Long
Private Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long
Private Declare Function ScreenToClient Lib "user32" (ByVal hWnd As Long, lpPoint As POINTAPI) As Long
Private Declare Function ClientToScreen Lib "user32" (ByVal hWnd As Long, lpPoint As POINTAPI) As Long
Private Declare Function ClipCursor Lib "user32" (lpRect As Any) As Long
Private Declare Function GetClientRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function SetCapture Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function ReleaseCapture Lib "user32" () As Long
Private Declare Function GetCapture Lib "user32" () As Long
Private Declare Function WindowFromPoint Lib "user32" (ByVal xPoint As Long, ByVal yPoint As Long) As Long

' Return the state of the Left button.

Function LeftButton() As Boolean
    LeftButton = (GetAsyncKeyState(vbKeyLButton) And &H8000)
End Function

' Return the state of the Right button.

Function RightButton() As Boolean
    RightButton = (GetAsyncKeyState(vbKeyRButton) And &H8000)
End Function

' Return the state of the Middle button.

Function MiddleButton() As Boolean
    MiddleButton = (GetAsyncKeyState(vbKeyMButton) And &H8000)
End Function

' Return the state of the mouse buttons as a bit-coded value.
' (same format as the Button argument of a Mousexxxx event procedure.)

Function MouseButton() As Integer
    If GetAsyncKeyState(vbKeyLButton) < 0 Then
        MouseButton = 1
    End If
    If GetAsyncKeyState(vbKeyRButton) < 0 Then
        MouseButton = MouseButton Or 2
    End If
    If GetAsyncKeyState(vbKeyMButton) < 0 Then
        MouseButton = MouseButton Or 4
    End If
End Function

' Return the number of mouse buttons.

Function MouseButtonsNumber() As Integer
    Const SM_CMOUSEBUTTONS = 43
    MouseButtonsNumber = GetSystemMetrics(SM_CMOUSEBUTTONS)
End Function

' Show and Hide the mouse cursor.
' (These calls must be balanced.)

Sub MouseHide()
    ShowCursor False
End Sub

Sub MouseShow()
    ShowCursor True
End Sub

' Get mouse X coordinates (screen coordinates).
' If a window handle is passed, the result are client area coordinates
' relative to that window.

Function MouseX(Optional ByVal hWnd As Long) As Long
    Dim lpPoint As POINTAPI
    GetCursorPos lpPoint
    If hWnd Then ScreenToClient hWnd, lpPoint
    MouseX = lpPoint.X
End Function

Function MouseY(Optional ByVal hWnd As Long) As Long
    Dim lpPoint As POINTAPI
    GetCursorPos lpPoint
    If hWnd Then ScreenToClient hWnd, lpPoint
    MouseY = lpPoint.Y
End Function

' Mouse mouse cursor. If a window handle is passed, the coordinates
' are taken as relative to that window's client area.

Sub MoveMouseCursor(ByVal X As Long, ByVal Y As Long, Optional ByVal hWnd As Long)
    If hWnd = 0 Then
        SetCursorPos X, Y
    Else
        Dim lpPoint As POINTAPI
        lpPoint.X = X
        lpPoint.Y = Y
        ClientToScreen hWnd, lpPoint
        SetCursorPos lpPoint.X, lpPoint.Y
    End If
End Sub

' Clip the mouse cursor to a given rectangle.

Sub ClipMouse(ByVal Left As Long, ByVal Top As Long, ByVal Right As Long, ByVal Bottom As Long)
    Dim lpRect As RECT
    lpRect.Left = Left
    lpRect.Top = Top
    lpRect.Right = Right
    lpRect.Bottom = Bottom
    ClipCursor lpRect
End Sub

' Clip mouse to a given window.

Sub ClipMouseToWindow(ByVal hWnd As Long)
    Dim lpPoint As POINTAPI, lpRect As RECT
    ' Retrieve the coordinates of the upper-left corner of the window.
    ClientToScreen hWnd, lpPoint
    ' Get the client screen rectangle.
    GetClientRect hWnd, lpRect
    ' Manually convert the rectangle's elements to screen coordinates.
    lpRect.Left = lpRect.Left + lpPoint.X
    lpRect.Top = lpRect.Top + lpPoint.Y
    lpRect.Right = lpRect.Right + lpPoint.X
    lpRect.Bottom = lpRect.Bottom + lpPoint.Y
    ' Enforce the clipping.
    ClipCursor lpRect
End Sub

' Clear the mouse clip area.

Sub ClearClipMouse()
    ClipCursor ByVal 0&
End Sub

' Set mouse capture.

Sub SetMouseCapture(ByVal hWnd As Long)
    If hWnd Then
        SetCapture hWnd
    Else
        ReleaseCapture
    End If
End Sub

' Get the handle of the window that has the mouse capture.

Function GetMouseCapture() As Long
    GetMouseCapture = GetCapture()
End Function

' Return the handle of the window under the mouse cursor.

Function WindowFromMouse() As Long
    Dim lpPoint As POINTAPI
    GetCursorPos lpPoint
    WindowFromMouse = WindowFromPoint(lpPoint.X, lpPoint.Y)
End Function
